#include <assert.h>
#include <malloc.h>
#include <stdio.h>
#include "sprite_types.h"
#include "control.h"
#include "maths.h"
#include "mech.h"
#include "airstrike.h"

static animation_t *anim;

static int setup()
{
  assert(anim = animation_load(path_to_data("missile.png"),
				     64,1,1000000));
  animation_make_loop(anim);
  return 0;
}

static sprite_t *create()
{
  sprite_t *s;
  s = calloc(1,sizeof(mech_sprite_t));
  s->animation = anim;
  ((mech_sprite_t *)s)->angle = 0;
  return s;
}

static void update(sprite_t *s)
{
  float n[2] = {0,1};
  mech_boundary((mech_sprite_t *)s);
  ((mech_sprite_t *)s)->lin_impulse[1] += 2;
  mech_turn_to((mech_sprite_t *)s,n,1);
  mech_update((mech_sprite_t *)s);
}

static void collide(struct sprite *this_sprite, 
		    struct sprite *other_sprite, 
		    int x, int y)
{
  sprite_t *s;
  float n[2];
  
  if (other_sprite->type == &missile) 
    return;
  s = sprite_create(&dsmoke);
  sprite_set_pos(s,x,y);
  sprite_get_vel(this_sprite, n);
  s->anim_p = ((vangle(n) + 128) & 255)/8 ;
  sprite_get_vel(other_sprite,n);
  sprite_set_vel(s,n);
  sprite_group_insert(effects_group,s);
  sprite_kill(this_sprite);
}

static void collide_world(struct sprite *this_sprite, int x, int y)
{
  sprite_t *s;
  
  s = sprite_create(&explosion);
  sprite_set_pos(s,x,y);
  sprite_group_insert(effects_group,s);  
  sprite_kill(this_sprite);
}

sprite_type_t missile =
  {
    "missile",
    setup,
    create,
    sprite_free,
    collide,
    collide_world,
    update
  };

REGISTER_SPRITE_TYPE(missile)
