#include <assert.h>
#include <malloc.h>
#include <stdio.h>
#include "sprite_types.h"
#include "airstrike.h"
#include "maths.h"
#include "mech.h"

static animation_t *anim;

static void frame_trigger(sprite_t *s)
{
  sprite_set_animation(s,anim);
  s->state = 0;
}

static void killme(sprite_t *s)
{
  sprite_kill(s);
}

static int setup()
{
  assert(anim = animation_load(path_to_data("zeppelin.png"),
			       1,8,300));
  animation_make_loop(anim);
  return 0;
}

static sprite_t *create()
{
  sprite_t *s;
  s = calloc(1,sizeof(mech_sprite_t));
  s->animation = anim;
  s->anim_p=0;
  mech_defaults((mech_sprite_t *)s,0);
  ((mech_sprite_t *)s)->rmass = 0.1; /*very heavy */
  ((mech_sprite_t *)s)->gravity = 0; /* floats */
  return s;
}

static void update(sprite_t *s)
{
  mech_boundary((mech_sprite_t *)s);
  mech_update((mech_sprite_t *)s);
  if (((mech_sprite_t *)s)->damage > 50)
    {
      create_effect(&explosion,s->x - 30, s->y - 15);
      create_effect(&explosion,s->x + 25, s->y - 6);
      create_effect(&explosion,s->x + 10, s->y + 14);
      sprite_kill(s);
    }
}

static void sigget(sprite_t *s, int signal, void *data)
{
  switch(signal)
    {
    case SIGNAL_DAMAGE:
      ((mech_sprite_t *)s)->damage += *(int *)data;
      break;
    default:
      break;
    }
}

sprite_type_t zeppelin =
  {
    "zeppelin",
    setup,
    create,
    sprite_free,
    0,
    0,
    update,
    sigget
  };

REGISTER_SPRITE_TYPE(zeppelin)
