#include <assert.h>
#include <SDL_image.h>
#include "text.h"
#include "sprite.h"
#include "config.h"

static SDL_Surface *chars[256];
static char *filechars = "!\"#$%&'()*+,-./0123456789:;<=>?@ABCDEFGHIJKLMNOPQRSTUVWXYZ[]^_`abcdefghijklmnopqrstuvwxyz{|}~ ";
static int char_h = 28; /* char height */
static int char_w = 15; /* Actual char width */
static int char_d = 25; /* Distance between two chars */
int text_char_w = 15;
int text_char_h = 28;

int text_setup()
{
  SDL_Surface *src;
  SDL_Rect r;
  char *c;
  r.x = 0;
  r.y = 0;
  r.w = char_w;
  r.h = char_h;
  assert(src = IMG_Load(path_to_data("font.png")));
  for (c=filechars;*c;c++)
    {
      GetSubSurface(src,&r,&(chars[(unsigned char)*c]),0,
		    0,0,SDL_RLEACCEL,0);
      r.x += char_d;
    }
  return 0;
}

void text_render(SDL_Surface *target, SDL_Rect *r,
		 int x, int y, 
		 enum text_alignment xalign, 
		 enum text_alignment yalign,
		 char *s)
{
  int w = 0,h = char_h; /* How big rect we are going to cover */
  int rw = 0;
  int xstart,ystart;
  char *c = s;
  SDL_Rect t;
  while (*c)
    {
      if (*c == '\n')
	{
	  if (rw > w)
	    w = rw;
	  rw = 0;
	  if (*(c+1))
	    h += char_h;
	}
      else
	rw += char_w;
      c++;
    }
  if (rw > w)
    w = rw;

  if (xalign == ALIGN_LEFT)
    xstart = x;
  else if (xalign == ALIGN_RIGHT)
    xstart = x - w;
  else
    xstart = x - w/2;

  if (yalign == ALIGN_TOP)
    ystart = y;
  else if (yalign == ALIGN_BOTTOM)
    ystart = y - h;
  else
    ystart = y - h/2;

  if (r)
    {
      r->x = xstart;
      r->y = ystart;
      r->w = w;
      r->h = h;
    }
  t.x = xstart;
  t.y = ystart;
  t.w = char_w;
  t.h = char_h;
  /* Now we can finally render */  
  for (c = s; *c != 0; c++)
    {
      if (*c == '\n')
	{
	  t.x = xstart;
	  t.y += char_h;
	}
      else if (chars[(unsigned char)*c])
	{
	  SDL_BlitSurface(chars[(unsigned char)*c],0,target,&t);
	  t.x += char_w;
	}
      else
	{
	  t.x += char_w;
	}
    }
  /* clip and mark dirty */
  if (xstart + w > target->w)
    w = target->w - xstart;
  if (ystart + h > target->h)
    h = target->h - ystart;
  if (xstart < 0)
    xstart = 0;
  if (ystart < 0)
    ystart = 0;
  t.x = xstart;
  t.y = ystart;
  t.w = w;
  t.h = h;
  sprite_dirty(&t);
}

